#ifndef BUFFER_H
#define BUFFER_H

#include <cstddef>

class Buffer {
	
	private:
	char *data;
	unsigned long length, blocksize;
	unsigned long pos;
	bool error;
	
	public:
	Buffer();
	~Buffer();
	void Clear();
	void SetPos(unsigned long newpos);
	void SetLength(unsigned long newlength);
	void Swap(Buffer* other);
	
	bool ReadFromFile(const char* filename);
	bool WriteToFile(const char* filename);
	
	template<typename A> A ReadType();
	template<typename A> void WriteType(A x);
	const char* ReadString();
	void WriteString(const char* string);
	void ReadData(char* ptr, unsigned long len);
	void WriteData(const char* ptr, unsigned long len);
	void ReadHex(char* ptr, unsigned long len);
	void WriteHex(const char* ptr, unsigned long len);
	
	void WriteBuffer(const Buffer* b);
	void WriteBufferPart(const Buffer* b, unsigned long _pos, unsigned long _len);
	
	public:
	inline const char* ToString() { return (data==NULL)? "" : data; }
	inline char* GetData() { return data; }
	inline unsigned long GetPos() { return pos; }
	inline unsigned long GetLength() { return length; }
	//inline char& operator[](unsigned long i) { return data[i]; }
	inline bool IsAtEnd() { return (pos>=length); }
	inline bool GetError() { return error; }
	inline void ClearError() { error = false; }
	
};

template<typename A>
A Buffer::ReadType() {
	if(error) return A();
	if(pos+sizeof(A)>length) {
		error = true;
		return A();
	}
	A *a = (A*)(data+pos);
	pos += sizeof(A);
	return *a;
}
template<typename A>
void Buffer::WriteType(A x) {
	if(sizeof(A)!=0) {
		unsigned long p = length;
		SetLength(length+sizeof(A));
		*((A*)(data+p)) = x;
	}
}

#endif // BUFFER_H

