#ifndef HTTPREQUEST_H
#define HTTPREQUEST_H

#include "Socket.h"
#include <string>
#include <vector>

#define HTTPREQUESTSTATE_NOTCONNECTED      0
#define HTTPREQUESTSTATE_WAIT_STATUSCODE   1
#define HTTPREQUESTSTATE_WAIT_HEADERS      2
#define HTTPREQUESTSTATE_WAIT_MESSAGEBODY  3
#define HTTPREQUESTSTATE_CLOSED            4
#define HTTPREQUESTSTATE_ERROR             5

class HttpRequest {
	
	private:
	struct Header {
		std::string name, value;
		inline Header() {}
		inline Header(const char* _name, const char* _value) : name(_name), value(_value) {}
	};
	struct PostParameter {
		std::string name, filename, value;
		inline PostParameter() {}
		inline PostParameter(const char* _name, const char* _filename, const char* _value) : name(_name), filename(_filename), value(_value) {}
		inline PostParameter(const char* _name, const char* _filename, const char* _value, unsigned int _valuelength) : name(_name), filename(_filename), value(_value, _valuelength) {}
	};
	
	private:
	int state;
	Socket s;
	std::vector<Header> headers;
	std::vector<PostParameter> postparameters;
	unsigned int statuscode;
	
	private:
	bool HeaderExists(const char* name);
	
	public:
	HttpRequest();
	
	void SetHeader(const char* name, const char* value, bool replace = true);
	void RemoveHeader(const char* name);
	void ClearHeaders();
	
	void SetPostParameter(const char* name, const char* value);
	void SetPostParameterFile(const char* name, const char* filename, const char* value, unsigned int valuelength);
	void RemovePostParameter(const char* name);
	void ClearPostParameters();
	
	void Connect(const char* _url, bool _post);
	void Update();
	void Reset();
	
	unsigned int GetStatusCode();
	
	unsigned int GetHeaderCount();
	const char* GetHeaderName(unsigned int i);
	const char* GetHeaderValue(unsigned int i);
	const char* GetHeader(const char* name);
	
	const char* GetMessageBody();
	unsigned long GetMessageBodyLength();
	
	static std::string UrlEncode(const char* str, bool keepspecialchars = false);
	
	public:
	inline int GetState() { return state; }
	
};

#endif // HTTPREQUEST_H

