#ifndef SOCKET_H
#define SOCKET_H

#define WINVER          0x0501
#define _WIN32_WINNT    0x0501
#define _WIN32_WINDOWS  0x0501
#define _WIN32_IE       0x0501

#include <winsock2.h>
#include <ws2tcpip.h>

#define LISTENINGSOCKETSTATE_NOTLISTENING  0
#define LISTENINGSOCKETSTATE_LISTENING     1
#define LISTENINGSOCKETSTATE_ERROR         2

#define SOCKETSTATE_NOTCONNECTED  0
#define SOCKETSTATE_CONNECTING    1
#define SOCKETSTATE_CONNECTED     2
#define SOCKETSTATE_SHUTDOWN      3
#define SOCKETSTATE_CLOSED        4
#define SOCKETSTATE_ERROR         5

bool InitSockets();

class ListeningSocket;
class Socket;

class ListeningSocket {
	
	private:
	int state;
	SOCKET s;
	
	public:
	ListeningSocket();
	~ListeningSocket();
	void Create(bool ipv6, const char* port);
	void Destroy();
	bool CanAccept();
	void Accept(Socket* socket);
	
	public:
	inline int GetState() { return state; }
	
};

class Socket {
	
	friend class ListeningSocket;
	
	private:
	int state;
	addrinfo *first_addrinfo, *current_addrinfo;
	SOCKET s;
	char *readbuffer, *writebuffer;
	unsigned long readstart, readlen, readblocksize, writestart, writelen, writeblocksize;
	
	private:
	void Init();
	void ResizeReadBuffer(unsigned long length);
	void ResizeWriteBuffer(unsigned long length);
	
	public:
	Socket();
	~Socket();
	void Create(const char* address, const char* port);
	void Destroy();
	bool Update();
	void ShutDown();
	const char* GetPeerAddress();
	
	const char* GetReadData();
	unsigned long GetReadDataLength();
	void EraseReadData(unsigned long length);
	
	void Write(const char* data, unsigned long length);
	char* Write(unsigned long length);
	unsigned long GetWriteDataLength();
	
	public:
	inline int GetState() { return state; }
	
};

#endif // SOCKET_H

