#ifndef GM_H
#define GM_H

#include "md5.h"
#include "Buffer.h"
#include "Socket.h"
#include "HttpRequest.h"

#include <climits>
#include <cstring>
#include <float.h>
#include <stdint.h>
#include <vector>
#include <string>

#define gmexport extern "C" __declspec (dllexport)

inline bool gm_double_to_bool(double d) {
	return (d>=0.5);
}
inline long gm_double_to_long(double d) {
	if(d<=(double)(long)(LONG_MIN)) return (long)(LONG_MIN);
	if(d>=(double)(long)(LONG_MAX)) return (long)(LONG_MAX);
	return (long)(d);
}
inline unsigned long gm_double_to_ulong(double d) {
	if(d<=(double)(unsigned long)(0)) return (unsigned long)(0);
	if(d>=(double)(unsigned long)(ULONG_MAX)) return (unsigned long)(ULONG_MAX);
	return (unsigned long)(d);
}
inline float gm_double_to_float(double d) {
	if(d<=(double)(float)(-FLT_MAX)) return (float)(-FLT_MAX);
	if(d>=(double)(float)(+FLT_MAX)) return (float)(+FLT_MAX);
	return (float)(d);
}

inline const char* ulong_to_str(char buffer[21], unsigned long x) {
	buffer[20] = '\0';
	char *ptr = buffer+20;
	do {
		*(--ptr) = '0'+x%10;;
	} while((x /= 10)!=0);
	return ptr;
}

struct GMData_Buffer {
	unsigned long id;
	Buffer *ptr;
};
struct GMData_ListeningSocket {
	unsigned long id;
	ListeningSocket *ptr;
};
struct GMData_Socket {
	unsigned long id;
	Socket *ptr;
};
struct GMData_HttpRequest {
	unsigned long id;
	HttpRequest *ptr;
};

struct GMData {
	
	unsigned long idcounter_buffers, idcounter_listeningsockets, idcounter_sockets, idcounter_httprequests;
	std::vector<GMData_Buffer> buffers;
	std::vector<GMData_ListeningSocket> listeningsockets;
	std::vector<GMData_Socket> sockets;
	std::vector<GMData_HttpRequest> httprequests;
	
	GMData() {
		idcounter_buffers = 0;
		idcounter_listeningsockets = 0;
		idcounter_sockets = 0;
		idcounter_httprequests = 0;
		InitSockets();
	}
	~GMData() {
		for(unsigned int i = 0; i<buffers.size(); ++i) {
			delete buffers[i].ptr;
		}
		for(unsigned int i = 0; i<listeningsockets.size(); ++i) {
			delete listeningsockets[i].ptr;
		}
		for(unsigned int i = 0; i<sockets.size(); ++i) {
			delete sockets[i].ptr;
		}
		for(unsigned int i = 0; i<httprequests.size(); ++i) {
			delete httprequests[i].ptr;
		}
	}
	inline Buffer* FindBuffer(unsigned long id) {
		for(unsigned int i = 0; i<buffers.size(); ++i) {
			if(buffers[i].id==id) return buffers[i].ptr;
		}
		return NULL;
	}
	inline ListeningSocket* FindListeningSocket(unsigned long id) {
		for(unsigned int i = 0; i<listeningsockets.size(); ++i) {
			if(listeningsockets[i].id==id) return listeningsockets[i].ptr;
		}
		return NULL;
	}
	inline Socket* FindSocket(unsigned long id) {
		for(unsigned int i = 0; i<sockets.size(); ++i) {
			if(sockets[i].id==id) return sockets[i].ptr;
		}
		return NULL;
	}
	inline HttpRequest* FindHttpRequest(unsigned long id) {
		for(unsigned int i = 0; i<httprequests.size(); ++i) {
			if(httprequests[i].id==id) return httprequests[i].ptr;
		}
		return NULL;
	}
	
};

extern GMData gmdata;
extern Buffer returnstring;

#endif // GM_H

