#include "gm.h"

gmexport double buffer_create() {
	Buffer *b = NULL; // GCC is complaining for some reason ...
	b = new Buffer();
	try {
		GMData_Buffer temp;
		temp.id = ++(gmdata.idcounter_buffers);
		temp.ptr = b;
		gmdata.buffers.push_back(temp);
	}
	catch(...) {
		delete b;
		throw;
	}
	return gmdata.idcounter_buffers;
}
gmexport double buffer_destroy(double id) {
	unsigned long ul_id = gm_double_to_ulong(id);
	for(unsigned int i = 0; i<gmdata.buffers.size(); ++i) {
		if(gmdata.buffers[i].id==ul_id) {
			Buffer *b = gmdata.buffers[i].ptr;
			gmdata.buffers.erase(gmdata.buffers.begin()+i);
			delete b;
			return 1;
		}
	}
	return 0;
}
gmexport double buffer_exists(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	return (b==NULL)? 0 : 1;
}

gmexport const char* buffer_to_string(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ToString();
}
gmexport double buffer_get_pos(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->GetPos();
}
gmexport double buffer_get_length(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->GetLength();
}
gmexport double buffer_at_end(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return (b->IsAtEnd())? 1 : 0;
}
gmexport double buffer_get_error(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return (b->GetError())? 1 : 0;
}
gmexport double buffer_clear_error(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->ClearError();
	return 1;
}

gmexport double buffer_clear(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->Clear();
	return 1;
}
gmexport double buffer_set_pos(double id, double pos) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->SetPos(gm_double_to_ulong(pos));
	return 1;
}

gmexport double buffer_read_from_file(double id, const char* filename) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return (b->ReadFromFile(filename))? 1 : 0;
}
gmexport double buffer_write_to_file(double id, const char* filename) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return (b->WriteToFile(filename))? 1 : 0;
}

gmexport double buffer_read_int8(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<int8_t>();
}
gmexport double buffer_read_uint8(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<uint8_t>();
}
gmexport double buffer_read_int16(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<int16_t>();
}
gmexport double buffer_read_uint16(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<uint16_t>();
}
gmexport double buffer_read_int32(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<int32_t>();
}
gmexport double buffer_read_uint32(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<uint32_t>();
}
gmexport double buffer_read_int64(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<int64_t>();
}
gmexport double buffer_read_uint64(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<uint64_t>();
}
gmexport double buffer_read_float32(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<float>();
}
gmexport double buffer_read_float64(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	return b->ReadType<double>();
}

gmexport double buffer_write_int8(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<int8_t>(gm_double_to_long(value));
	return 1;
}
gmexport double buffer_write_uint8(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<uint8_t>(gm_double_to_ulong(value));
	return 1;
}
gmexport double buffer_write_int16(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<int16_t>(gm_double_to_long(value));
	return 1;
}
gmexport double buffer_write_uint16(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<uint16_t>(gm_double_to_ulong(value));
	return 1;
}
gmexport double buffer_write_int32(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<int32_t>(gm_double_to_long(value));
	return 1;
}
gmexport double buffer_write_uint32(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<uint32_t>(gm_double_to_ulong(value));
	return 1;
}
gmexport double buffer_write_int64(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<int64_t>(gm_double_to_long(value));
	return 1;
}
gmexport double buffer_write_uint64(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<uint64_t>(gm_double_to_ulong(value));
	return 1;
}
gmexport double buffer_write_float32(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<float>(gm_double_to_float(value));
	return 1;
}
gmexport double buffer_write_float64(double id, double value) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteType<double>(value);
	return 1;
}

gmexport const char* buffer_read_string(double id) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return "";
	return b->ReadString();
}
gmexport double buffer_write_string(double id, const char* string) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteString(string);
	return 1;
}

gmexport const char* buffer_read_data(double id, double len) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return "";
	unsigned long l = gm_double_to_ulong(len);
	returnstring.Clear();
	returnstring.SetLength(l);
	b->ReadData(returnstring.GetData(), l);
	return returnstring.ToString();
}
gmexport double buffer_write_data(double id, const char* string) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteData(string, strlen(string));
	return 1;
}

gmexport const char* buffer_read_hex(double id, double len) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return "";
	unsigned long l = gm_double_to_ulong(len);
	returnstring.Clear();
	returnstring.SetLength(l*2);
	b->ReadHex(returnstring.GetData(), l);
	return returnstring.ToString();
}
gmexport double buffer_write_hex(double id, const char* string) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	b->WriteHex(string, strlen(string)/2);
	return 1;
}

gmexport double buffer_write_buffer(double id, double id2) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	Buffer *b2 = gmdata.FindBuffer(gm_double_to_ulong(id2));
	if(b2==NULL) return 0;
	b->WriteBuffer(b2);
	return 1;
}
gmexport double buffer_write_buffer_part(double id, double id2, double pos, double len) {
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(id));
	if(b==NULL) return 0;
	Buffer *b2 = gmdata.FindBuffer(gm_double_to_ulong(id2));
	if(b2==NULL) return 0;
	b->WriteBufferPart(b2, gm_double_to_ulong(pos), gm_double_to_ulong(len));
	return 1;
}

