#include "gm.h"

gmexport double httprequest_create() {
	HttpRequest *hr = NULL; // GCC is complaining for some reason ...
	hr = new HttpRequest();
	try {
		GMData_HttpRequest temp;
		temp.id = ++(gmdata.idcounter_httprequests);
		temp.ptr = hr;
		gmdata.httprequests.push_back(temp);
	}
	catch(...) {
		delete hr;
		throw;
	}
	return gmdata.idcounter_httprequests;
}
gmexport double httprequest_destroy(double id) {
	unsigned long ul_id = gm_double_to_ulong(id);
	for(unsigned int i = 0; i<gmdata.httprequests.size(); ++i) {
		if(gmdata.httprequests[i].id==ul_id) {
			HttpRequest *hr = gmdata.httprequests[i].ptr;
			gmdata.httprequests.erase(gmdata.httprequests.begin()+i);
			delete hr;
			return 1;
		}
	}
	return 0;
}
gmexport double httprequest_exists(double id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	return (hr==NULL)? 0 : 1;
}

gmexport double httprequest_get_state(double id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	return hr->GetState();
}

gmexport double httprequest_set_header(double id, const char* name, const char* value, double replace) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()!=HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->SetHeader(name, value, gm_double_to_bool(replace));
	return 1;
}
gmexport double httprequest_remove_header(double id, const char* name) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()!=HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->RemoveHeader(name);
	return 1;
}
gmexport double httprequest_clear_headers(double id, const char* name, const char* value) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()!=HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->ClearHeaders();
	return 1;
}

gmexport double httprequest_set_post_parameter(double id, const char* name, const char* value) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()!=HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->SetPostParameter(name, value);
	return 1;
}
gmexport double httprequest_set_post_parameter_file(double id, const char* name, const char* filename, double buffer_id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(buffer_id));
	if(b==NULL) return 0;
	if(hr->GetState()!=HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->SetPostParameterFile(name, filename, b->GetData(), b->GetLength());
	return 1;
}
gmexport double httprequest_remove_post_parameter(double id, const char* name) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()!=HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->RemovePostParameter(name);
	return 1;
}
gmexport double httprequest_clear_post_parameters(double id, const char* name, const char* value) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()!=HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->ClearPostParameters();
	return 1;
}

gmexport double httprequest_connect(double id, const char* url, double post) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()!=HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->Connect(url, gm_double_to_bool(post));
	return 1;
}
gmexport double httprequest_update(double id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->Update();
	return 1;
}
gmexport double httprequest_reset(double id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	hr->Reset();
	return 1;
}

gmexport double httprequest_get_status_code(double id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	return hr->GetStatusCode();
}

gmexport double httprequest_get_header_count(double id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	return hr->GetHeaderCount();
}
gmexport const char* httprequest_get_header_name(double id, double index) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return "";
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	const char *str = hr->GetHeaderName(gm_double_to_ulong(index));
	return (str==NULL)? "" : str;
}
gmexport const char* httprequest_get_header_value(double id, double index) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return "";
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	const char *str = hr->GetHeaderValue(gm_double_to_ulong(index));
	return (str==NULL)? "" : str;
}
gmexport const char* httprequest_get_header(double id, const char* name) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return "";
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	const char *str = hr->GetHeader(name);
	return (str==NULL)? "" : str;
}

gmexport const char* httprequest_get_message_body(double id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return "";
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	returnstring.Clear();
	returnstring.SetLength(hr->GetMessageBodyLength());
	memcpy(returnstring.GetData(), hr->GetMessageBody(), hr->GetMessageBodyLength());
	return returnstring.ToString();
}
gmexport double httprequest_get_message_body_length(double id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	return hr->GetMessageBodyLength();
}
gmexport double httprequest_get_message_body_buffer(double id, double buffer_id) {
	HttpRequest *hr = gmdata.FindHttpRequest(gm_double_to_ulong(id));
	if(hr==NULL) return 0;
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(buffer_id));
	if(b==NULL) return 0;
	if(hr->GetState()==HTTPREQUESTSTATE_NOTCONNECTED) return 0;
	b->Clear();
	b->SetLength(hr->GetMessageBodyLength());
	memcpy(b->GetData(), hr->GetMessageBody(), hr->GetMessageBodyLength());
	return 1;
}

gmexport const char* httprequest_urlencode(const char* string, double keepspecialchars) {
	std::string str = HttpRequest::UrlEncode(string, gm_double_to_bool(keepspecialchars));
	returnstring.Clear();
	returnstring.SetLength(str.length());
	memcpy(returnstring.GetData(), str.data(), str.length());
	return returnstring.ToString();
}

