#include "gm.h"

gmexport double socket_create(const char* address, double port) {
	Socket *s = NULL; // GCC is complaining for some reason ...
	s = new Socket();
	try {
		char buffer[21];
		s->Create(address, ulong_to_str(buffer, gm_double_to_ulong(port)));
		GMData_Socket temp;
		temp.id = ++(gmdata.idcounter_sockets);
		temp.ptr = s;
		gmdata.sockets.push_back(temp);
	}
	catch(...) {
		delete s;
		throw;
	}
	return gmdata.idcounter_sockets;
}
gmexport double socket_destroy(double id) {
	unsigned long ul_id = gm_double_to_ulong(id);
	for(unsigned int i = 0; i<gmdata.sockets.size(); ++i) {
		if(gmdata.sockets[i].id==ul_id) {
			Socket *s = gmdata.sockets[i].ptr;
			gmdata.sockets.erase(gmdata.sockets.begin()+i);
			delete s;
			return 1;
		}
	}
	return 0;
}
gmexport double socket_exists(double id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	return (s==NULL)? 0 : 1;
}

gmexport double socket_get_state(double id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	return s->GetState();
}

gmexport double socket_update(double id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	return (s->Update())? 1 : 0;

}
gmexport double socket_shut_down(double id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	if(s->GetState()==SOCKETSTATE_CONNECTING) return 0;
	s->ShutDown();
	return 1;
}
gmexport const char* socket_get_peer_address(double id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return "";
	if(s->GetState()==SOCKETSTATE_CONNECTING) return "";
	const char *str = s->GetPeerAddress();
	return (str==NULL)? "" : str;
}

gmexport double socket_read_message(double id, double buffer_id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(buffer_id));
	if(b==NULL) return 0;
	if(s->GetReadDataLength()<4) return 0;
	const char *data = s->GetReadData();
	uint32_t len = *((uint32_t*)(data));
	if(s->GetReadDataLength()<4+len) return 0;
	b->Clear();
	b->SetLength(len);
	memcpy(b->GetData(), data+4, len);
	s->EraseReadData(4+len);
	return 1;
}

gmexport double socket_read_message_delimiter(double id, double buffer_id, const char* delimiter) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(buffer_id));
	if(b==NULL) return 0;
	const char *data = s->GetReadData();
	unsigned long i;
	for(i = 0; i<s->GetReadDataLength(); ++i) {
		if(data[i]==delimiter[0]) break;
	}
	if(i>=s->GetReadDataLength()) {
		return 0;
	}
	b->Clear();
	b->SetLength(i);
	memcpy(b->GetData(), data, i);
	s->EraseReadData(i+1);
	return 1;
}

gmexport double socket_read_message_raw(double id, double buffer_id, double bytes) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(buffer_id));
	if(b==NULL) return 0;
	unsigned long len;
	if(bytes<0.0) {
		len = s->GetReadDataLength();
	} else {
		len = gm_double_to_ulong(bytes);
		if(s->GetReadDataLength()<len) {
			return 0;
		}
	}
	b->Clear();
	b->SetLength(len);
	memcpy(b->GetData(), s->GetReadData(), len);
	s->EraseReadData(len);
	return 1;
}

gmexport double socket_write_message(double id, double buffer_id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(buffer_id));
	if(b==NULL) return 0;
	unsigned long len = b->GetLength();
	s->Write((const char*)(&len), 4);
	s->Write(b->GetData(), len);
	return 1;
}

gmexport double socket_write_message_delimiter(double id, double buffer_id, const char* delimiter) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(buffer_id));
	if(b==NULL) return 0;
	s->Write(b->GetData(), b->GetLength());
	s->Write(delimiter, 1);
	return 1;
}

gmexport double socket_write_message_raw(double id, double buffer_id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	Buffer *b = gmdata.FindBuffer(gm_double_to_ulong(buffer_id));
	if(b==NULL) return 0;
	s->Write(b->GetData(), b->GetLength());
	return 1;
}

gmexport double socket_get_read_data_length(double id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	return s->GetReadDataLength();
}

gmexport double socket_get_write_data_length(double id) {
	Socket *s = gmdata.FindSocket(gm_double_to_ulong(id));
	if(s==NULL) return 0;
	return s->GetWriteDataLength();
}

