#include "md5.h"
#include <cstdio>
#include <cstdlib>
#include <cstring>

/**
 *  Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
 *  rights reserved.
 *  
 *  License to copy and use this software is granted provided that it
 *  is identified as the "RSA Data Security, Inc. MD5 Message-Digest
 *  Algorithm" in all material mentioning or referencing this software
 *  or this function.
 *  
 *  License is also granted to make and use derivative works provided
 *  that such works are identified as "derived from the RSA Data
 *  Security, Inc. MD5 Message-Digest Algorithm" in all material
 *  mentioning or referencing the derived work.
 *  
 *  RSA Data Security, Inc. makes no representations concerning either
 *  the merchantability of this software or the suitability of this
 *  software for any particular purpose. It is provided "as is"
 *  without express or implied warranty of any kind.
 *  
 *  These notices must be retained in any copies of any part of this
 *  documentation and/or software.
**/

#define md5_S11 7
#define md5_S12 12
#define md5_S13 17
#define md5_S14 22
#define md5_S21 5
#define md5_S22 9
#define md5_S23 14
#define md5_S24 20
#define md5_S31 4
#define md5_S32 11
#define md5_S33 16
#define md5_S34 23
#define md5_S41 6
#define md5_S42 10
#define md5_S43 15
#define md5_S44 21

#define md5_F(x,y,z) (((x)&(y))|((~x)&(z)))
#define md5_G(x,y,z) (((x)&(z))|((y)&(~z)))
#define md5_H(x,y,z) ((x)^(y)^(z))
#define md5_I(x,y,z) ((y)^((x)|(~z)))

#define md5_ROTATE_LEFT(x,n) (((x) << (n)) | ((x) >> (32-n)))

#define md5_FF(a, b, c, d, x, s, ac) { \
	a += md5_F(b, c, d) + x + (uint32_t)(ac); \
	a = md5_ROTATE_LEFT(a, s); \
	a += b; \
}
#define md5_GG(a, b, c, d, x, s, ac) { \
	a += md5_G(b, c, d) + x + (uint32_t)(ac); \
	a = md5_ROTATE_LEFT(a, s); \
	a += b; \
}
#define md5_HH(a, b, c, d, x, s, ac) { \
	a += md5_H(b, c, d) + x + (uint32_t)(ac); \
	a = md5_ROTATE_LEFT(a, s); \
	a += b; \
}
#define md5_II(a, b, c, d, x, s, ac) { \
	a += md5_I(b, c, d) + x + (uint32_t)(ac); \
	a = md5_ROTATE_LEFT(a, s); \
	a += b; \
}

// initialize a md5 hash
void md5_init(uint32_t result[4]) {
	result[0] = 0x67452301;
	result[1] = 0xefcdab89;
	result[2] = 0x98badcfe;
	result[3] = 0x10325476;
}

// add 16 words to hash
void md5_hash(uint32_t result[4], uint32_t block[16]) {
	
	uint32_t a = result[0];
	uint32_t b = result[1];
	uint32_t c = result[2];
	uint32_t d = result[3];
	
	/* Round 1 */
	md5_FF(a, b, c, d, block[ 0], md5_S11, 0xd76aa478); /* 1 */
	md5_FF(d, a, b, c, block[ 1], md5_S12, 0xe8c7b756); /* 2 */
	md5_FF(c, d, a, b, block[ 2], md5_S13, 0x242070db); /* 3 */
	md5_FF(b, c, d, a, block[ 3], md5_S14, 0xc1bdceee); /* 4 */
	md5_FF(a, b, c, d, block[ 4], md5_S11, 0xf57c0faf); /* 5 */
	md5_FF(d, a, b, c, block[ 5], md5_S12, 0x4787c62a); /* 6 */
	md5_FF(c, d, a, b, block[ 6], md5_S13, 0xa8304613); /* 7 */
	md5_FF(b, c, d, a, block[ 7], md5_S14, 0xfd469501); /* 8 */
	md5_FF(a, b, c, d, block[ 8], md5_S11, 0x698098d8); /* 9 */
	md5_FF(d, a, b, c, block[ 9], md5_S12, 0x8b44f7af); /* 10 */
	md5_FF(c, d, a, b, block[10], md5_S13, 0xffff5bb1); /* 11 */
	md5_FF(b, c, d, a, block[11], md5_S14, 0x895cd7be); /* 12 */
	md5_FF(a, b, c, d, block[12], md5_S11, 0x6b901122); /* 13 */
	md5_FF(d, a, b, c, block[13], md5_S12, 0xfd987193); /* 14 */
	md5_FF(c, d, a, b, block[14], md5_S13, 0xa679438e); /* 15 */
	md5_FF(b, c, d, a, block[15], md5_S14, 0x49b40821); /* 16 */
	
	/* Round 2 */
	md5_GG(a, b, c, d, block[ 1], md5_S21, 0xf61e2562); /* 17 */
	md5_GG(d, a, b, c, block[ 6], md5_S22, 0xc040b340); /* 18 */
	md5_GG(c, d, a, b, block[11], md5_S23, 0x265e5a51); /* 19 */
	md5_GG(b, c, d, a, block[ 0], md5_S24, 0xe9b6c7aa); /* 20 */
	md5_GG(a, b, c, d, block[ 5], md5_S21, 0xd62f105d); /* 21 */
	md5_GG(d, a, b, c, block[10], md5_S22, 0x02441453); /* 22 */
	md5_GG(c, d, a, b, block[15], md5_S23, 0xd8a1e681); /* 23 */
	md5_GG(b, c, d, a, block[ 4], md5_S24, 0xe7d3fbc8); /* 24 */
	md5_GG(a, b, c, d, block[ 9], md5_S21, 0x21e1cde6); /* 25 */
	md5_GG(d, a, b, c, block[14], md5_S22, 0xc33707d6); /* 26 */
	md5_GG(c, d, a, b, block[ 3], md5_S23, 0xf4d50d87); /* 27 */
	md5_GG(b, c, d, a, block[ 8], md5_S24, 0x455a14ed); /* 28 */
	md5_GG(a, b, c, d, block[13], md5_S21, 0xa9e3e905); /* 29 */
	md5_GG(d, a, b, c, block[ 2], md5_S22, 0xfcefa3f8); /* 30 */
	md5_GG(c, d, a, b, block[ 7], md5_S23, 0x676f02d9); /* 31 */
	md5_GG(b, c, d, a, block[12], md5_S24, 0x8d2a4c8a); /* 32 */
	
	/* Round 3 */
	md5_HH(a, b, c, d, block[ 5], md5_S31, 0xfffa3942); /* 33 */
	md5_HH(d, a, b, c, block[ 8], md5_S32, 0x8771f681); /* 34 */
	md5_HH(c, d, a, b, block[11], md5_S33, 0x6d9d6122); /* 35 */
	md5_HH(b, c, d, a, block[14], md5_S34, 0xfde5380c); /* 36 */
	md5_HH(a, b, c, d, block[ 1], md5_S31, 0xa4beea44); /* 37 */
	md5_HH(d, a, b, c, block[ 4], md5_S32, 0x4bdecfa9); /* 38 */
	md5_HH(c, d, a, b, block[ 7], md5_S33, 0xf6bb4b60); /* 39 */
	md5_HH(b, c, d, a, block[10], md5_S34, 0xbebfbc70); /* 40 */
	md5_HH(a, b, c, d, block[13], md5_S31, 0x289b7ec6); /* 41 */
	md5_HH(d, a, b, c, block[ 0], md5_S32, 0xeaa127fa); /* 42 */
	md5_HH(c, d, a, b, block[ 3], md5_S33, 0xd4ef3085); /* 43 */
	md5_HH(b, c, d, a, block[ 6], md5_S34, 0x04881d05); /* 44 */
	md5_HH(a, b, c, d, block[ 9], md5_S31, 0xd9d4d039); /* 45 */
	md5_HH(d, a, b, c, block[12], md5_S32, 0xe6db99e5); /* 46 */
	md5_HH(c, d, a, b, block[15], md5_S33, 0x1fa27cf8); /* 47 */
	md5_HH(b, c, d, a, block[ 2], md5_S34, 0xc4ac5665); /* 48 */
	
	/* Round 4 */
	md5_II(a, b, c, d, block[ 0], md5_S41, 0xf4292244); /* 49 */
	md5_II(d, a, b, c, block[ 7], md5_S42, 0x432aff97); /* 50 */
	md5_II(c, d, a, b, block[14], md5_S43, 0xab9423a7); /* 51 */
	md5_II(b, c, d, a, block[ 5], md5_S44, 0xfc93a039); /* 52 */
	md5_II(a, b, c, d, block[12], md5_S41, 0x655b59c3); /* 53 */
	md5_II(d, a, b, c, block[ 3], md5_S42, 0x8f0ccc92); /* 54 */
	md5_II(c, d, a, b, block[10], md5_S43, 0xffeff47d); /* 55 */
	md5_II(b, c, d, a, block[ 1], md5_S44, 0x85845dd1); /* 56 */
	md5_II(a, b, c, d, block[ 8], md5_S41, 0x6fa87e4f); /* 57 */
	md5_II(d, a, b, c, block[15], md5_S42, 0xfe2ce6e0); /* 58 */
	md5_II(c, d, a, b, block[ 6], md5_S43, 0xa3014314); /* 59 */
	md5_II(b, c, d, a, block[13], md5_S44, 0x4e0811a1); /* 60 */
	md5_II(a, b, c, d, block[ 4], md5_S41, 0xf7537e82); /* 61 */
	md5_II(d, a, b, c, block[11], md5_S42, 0xbd3af235); /* 62 */
	md5_II(c, d, a, b, block[ 2], md5_S43, 0x2ad7d2bb); /* 63 */
	md5_II(b, c, d, a, block[ 9], md5_S44, 0xeb86d391); /* 64 */
	
	result[0] += a;
	result[1] += b;
	result[2] += c;
	result[3] += d;
	a = 0;
	b = 0;
	c = 0;
	d = 0;
	
}

void MD5::Begin() {
	blocks = 0;
	used = 0;
	md5_init(result.w);
}
bool MD5::ReadFile(const char* fname) {
	FILE* f;
	f = fopen(fname, "rb");
	if(f==NULL) return false;
	for(;;) {
		used += fread(block.x+used, 1, 64-used, f);
		if(used<64) break;
		md5_hash(result.w, block.w);
		++blocks;
		used = 0;
	}
	fclose(f);
	return true;
}
void MD5::ReadMem(const void* data, unsigned long datalen) {
	if(datalen==0) return;
	unsigned long j = 0;
	uint8_t* in = (uint8_t*)(data);
	while(datalen-j>=64-used) {
		memcpy(block.x+used, in+j, 64-used);
		j += 64-used;
		md5_hash(result.w, block.w);
		++blocks;
		used = 0;
	}
	if(datalen-j>0) {
		memcpy(block.x+used, in+j, datalen-j);
		used += datalen-j;
	}
}
void MD5::End() {
	block.x[used] = 0x80;
	if(64-used>=8+1) {
		memset(block.x+used+1, 0, 64-used-1-8);
		block.w[14] = (blocks<<9)|(used<<3);
		block.w[15] = blocks>>(32-9);
		md5_hash(result.w, block.w);
	} else {
		memset(block.x+used+1, 0, 64-used-1);
		md5_hash(result.w, block.w);
		memset(block.x, 0, 64-8);
		block.w[14] = (blocks<<9)|(used<<3);
		block.w[15] = blocks>>(32-9);
		md5_hash(result.w, block.w);
	}
	memset(block.x, 0, 64);
}

void bintohex(const char* input, char* output, unsigned long inputlen) {
	for(unsigned long i = 0; i<inputlen; ++i) {
		unsigned char a = input[i];
		output[2*i]   = ((a>>4)>9)? 'a'+(a>>4)-10 : '0'+(a>>4);
		output[2*i+1] = ((a&15)>9)? 'a'+(a&15)-10 : '0'+(a&15);
	}
}

void hextobin(const char* input, char* output, unsigned long outputlen) {
	for(unsigned long i = 0; i<outputlen; ++i) {
		unsigned char a = input[2*i];
		unsigned char b = input[2*i+1];
		output[i] =
			((('0'<=a && a<='9')? a-'0' : (('a'<=a && a<='f')? 10+a-'a' : (('A'<=a && a<='F')? 10+a-'A' : 0)))<<4) |
			(('0'<=b && b<='9')? b-'0' : (('a'<=b && b<='f')? 10+b-'a' : (('A'<=b && b<='F')? 10+b-'A' : 0)));
	}
}

